@@ -1,10 +1,10 @@
-/*  $Revision: 1.10 $
+/*  $Revision: 1.16 $
 **
 **  A C version of Henry Spencer's "subst" script.
 */
 #include <stdio.h>
-#include <signal.h>
 #include <errno.h>
+#include <signal.h>
 
 #define LINESIZE		1024
 #define FNAMESIZE		1024
@@ -50,7 +50,9 @@ static char *
 xstrerror()
 {
     extern int	sys_nerr;
+#if !defined(__FreeBSD__) && !defined(__NetBSD__)
     extern char	*sys_errlist[];
+#endif
     extern int	errno;
     static char	buff[30];
 
@@ -326,13 +328,15 @@ doline(f, out, line, tp, end)
     char	*p;
     char	*new;
     char	save;
+    int		count;
 
-    for (line = xstrdup(line); tp < end; tp++) {
+    for (count = 0, line = xstrdup(line); tp < end; tp++) {
 Again:
 	for (p = line; (p = xstrchr(p, tp->Name[0])) != NULL; p++)
 	    if (strncmp(p, tp->Name, tp->Length) == 0) {
 		save = *p;
 		*p = '\0';
+		count++;
 		new = splice(line, tp->Value, p + tp->Length);
 		*p = save;
 		if (strcmp(new, line) == 0) {
@@ -346,14 +350,14 @@ Again:
 		goto Again;
 	    }
     }
-    if (fputs(line, out) == EOF) {
+    if (count > 0 && fputs(line, out) == EOF) {
 	(void)fprintf(stderr, "%s:  can't write %s, %s\n",
 		argv0, f, xstrerror());
 	free(line);
 	return -1;
     }
     free(line);
-    return 0;
+    return count;
 }
 
 
@@ -371,6 +375,7 @@ Process(f, Table, end)
     char	line[LINESIZE];
     int		bad;
     int		i;
+    int		count;
     FILE	*in;
     FILE	*out;
     FILE	*temp;
@@ -423,18 +428,26 @@ Process(f, Table, end)
     /* Generate the new version. */
     for (i = 1, bad = 0; fgets(line, sizeof line, in) != NULL; i++) {
 	if ((p = xstrchr(line, '\n')) == NULL) {
-	    (void)fprintf(stderr, "%s: Line %d of %s is too long\n",
+	    (void)fprintf(stderr,
+	      "%s: Line %d of %s is too long (or doesn't end with a newline)\n",
 		    argv0, i, f);
 	    bad++;
 	    break;
 	}
 	(void)fputs(line, out);
 	if ((p = istemplate(line)) != NULL) {
-	    if (doline(f, out, p, Table, end) < 0) {
+	    if ((count = doline(f, out, p, Table, end)) < 0) {
 		bad++;
 		break;
 	    }
-	    (void)fgets(line, sizeof line, in);
+	    if (count > 0) {
+		(void)fgets(line, sizeof line, in);
+		i++;
+	    }
+	    else
+		(void)fprintf(stderr,
+			"%s: %s:%d unknown parameter or bad line:\n\t%s",
+			argv0, f, i, p);
 	}
     }
     (void)fclose(in);
@@ -497,11 +510,13 @@ main(ac, av)
     argv0 = av[0];
 
     /* Parse JCL. */
-    while ((i = getopt(ac, av, "f:")) != EOF)
+    while ((i = getopt(ac, av, "e:f:")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
 	    /* NOTREACHED */
+	case 'e':
+	    break;
 	case 'f':
 	    if (ctlfile != NULL)
 		Usage();
