@@ -1,11 +1,12 @@
-/*  $Revision: 1.4 $
+/*  $Revision: 1.9 $
 **
-**  Expire news articles.
+**  Get data from history database.
 */
-#include "configdata.h"
 #include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
+#include <ctype.h>
 #include <sys/stat.h>
 #if	defined(DO_NEED_TIME)
 #include <time.h>
@@ -14,7 +15,6 @@
 #include <errno.h>
 #include "paths.h"
 #include "libinn.h"
-#include "clibrary.h"
 #include "dbz.h"
 #include "macros.h"
 
@@ -41,9 +41,9 @@ GetName(F, buff, Againp)
     (void)strcpy(buff, SPOOL);
     p = &buff[STRLEN(SPOOL)];
     *p++ = '/';
-    *p++ = c;
+    *p++ = (char)c;
     while ((c = getc(F)) != EOF && c != ' ' && c != '\n')
-	*p++ = c == '.' ? '/' : c;
+	*p++ = (char)(c == '.' ? '/' : c);
     *p = '\0';
     *Againp = c != EOF && c != '\n';
     return TRUE;
@@ -52,11 +52,10 @@ GetName(F, buff, Againp)
 /*
 **  Given a DBZ value, seek to the right spot.
 */
-STATIC void
-HistorySeek(F, p, key)
+STATIC BOOL
+HistorySeek(F, p)
     register FILE	*F;
     register char	*p;
-    register char	*key;
 {
     register char	*dest;
     OFFSET_T		l;
@@ -67,18 +66,21 @@ HistorySeek(F, p, key)
 	*dest++ = *p++;
     if (fseek(F, l, SEEK_SET) == -1) {
 	(void)fprintf(stderr, "Can't seek to %ld, %s\n", l, strerror(errno));
-	exit(1);
+	return FALSE;
     }
 
     /* Move to the filename fields. */
     for (i = 2; (c = getc(F)) != EOF && c != '\n'; )
 	if (c == HIS_FIELDSEP && --i == 0)
 	    break;
-    if (c != HIS_FIELDSEP) {
-	(void)fprintf(stderr, "Bad text line for \"%s\", %s\n",
-		key, strerror(errno));
-	exit(1);
-    }
+    if (c != HIS_FIELDSEP)
+	/* Could have only two fields (if expired) so don't complain now.
+	 * (void)fprintf(stderr, "Bad text line for \"%s\", %s\n",
+	 *	key, strerror(errno));
+	 */
+	return FALSE;
+
+    return TRUE;
 }
 
 
@@ -161,15 +163,18 @@ IhaveSendme(History, What)
 	}
 
 	/* Sendme -- print a filename for the message. */
-	HistorySeek(F, value.dptr, key.dptr);
-	while (GetName(F, Name, &More)) {
-	    if (stat(Name, &Sb) >= 0) {
-		(void)printf("%s\n", Name);
-		break;
+	if (value.dptr == NULL)
+	    /* Doesn't exist. */
+	    continue;
+	if (HistorySeek(F, value.dptr))
+	    while (GetName(F, Name, &More)) {
+		if (stat(Name, &Sb) >= 0) {
+		    (void)printf("%s\n", Name);
+		    break;
+		}
+		if (!More)
+		    break;
 	    }
-	    if (!More)
-		break;
-	}
     }
 }
 
@@ -206,7 +211,7 @@ main(ac, av)
     What = '?';
 
     /* Parse JCL. */
-    while ((i = getopt(ac, av, "f:eilnqs")) != EOF)
+    while ((i = getopt(ac, av, "f:eiltnqs")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -217,14 +222,15 @@ main(ac, av)
 	case 'e':
 	case 'i':
 	case 'l':
+	case 't':
 	case 'n':
 	case 'q':
 	case 's':
 	    if (What != '?') {
-		(void)fprintf(stderr, "Only one [eilnqs] flag allowed.\n");
+		(void)fprintf(stderr, "Only one [eiltnqs] flag allowed.\n");
 		exit(1);
 	    }
-	    What = i;
+	    What = (char)i;
 	    break;
 	}
     ac -= optind;
@@ -255,7 +261,7 @@ main(ac, av)
     key.dptr = av[0];
     if (*key.dptr != '<') {
 	/* Add optional braces. */
-	key.dptr = NEW(char, 1 + strlen(av[0]) + 1);
+	key.dptr = NEW(char, 1 + strlen(av[0]) + 1 + 1);
 	(void)sprintf(key.dptr, "<%s>", av[0]);
     }
     for (p = key.dptr; *p; p++)
@@ -275,6 +281,12 @@ main(ac, av)
     if (What == 'q')
 	exit(0);
 
+    /* Just give offset into history file */
+    if (What == 't') {
+      (void)printf("%lu\n", *(long *) value.dptr);
+      exit(0);
+    }
+
     /* Open the text file, go to the entry. */
     if ((F = fopen(History, "r")) == NULL) {
 	(void)fprintf(stderr, "Can't open \"%s\", %s\n",
@@ -285,17 +297,17 @@ main(ac, av)
 	FullLine(F, value.dptr);
 	exit(0);
     }
-    HistorySeek(F, value.dptr, key.dptr);
 
     /* Loop until we find an existing file. */
-    while (GetName(F, Name, &More)) {
-	if (stat(Name, &Sb) >= 0) {
-	    (void)printf("%s\n", Name);
-	    exit(0);
+    if (HistorySeek(F, value.dptr))
+	while (GetName(F, Name, &More)) {
+	    if (stat(Name, &Sb) >= 0) {
+		(void)printf("%s\n", Name);
+		exit(0);
+	    }
+	    if (!More)
+		break;
 	}
-	if (!More)
-	    break;
-    }
 
     if (What == 'n')
 	(void)printf("/dev/null\n");
