@@ -1,4 +1,4 @@
-/*  $Revision: 1.16 $
+/*  $Revision: 1.17 $
 **
 */
 #include <stdio.h>
@@ -21,6 +21,10 @@
 extern unsigned short	ntohs();
 #endif	/* !defined(ntohs) && !defined(NETSWAP) */
 
+#if	!defined(htons) && !defined(NETSWAP)
+extern unsigned short	htons();
+#endif	/* !defined(ntohs) && !defined(NETSWAP) */
+
 char *nntp_port = NULL;
 
 /*
@@ -80,9 +84,9 @@ NNTPconnect(p, FromServerp, ToServerp, errbuff)
     (void)memset((POINTER)&server, 0, sizeof server);
     server.sin_family = hp->h_addrtype;
     if (nntp_port)
- 	server.sin_port = ntohs(atoi(nntp_port));
+ 	server.sin_port = htons(atoi(nntp_port));
     else
-	server.sin_port = ntohs(NNTP_PORT);
+	server.sin_port = htons(NNTP_PORT);
   
     /* Loop through the address list, trying to connect. */
     for (; ap && *ap; ap++) {
