@@ -219,6 +219,9 @@ NNTPtoGMT(av1, av2)
     else if (hour < 0 || hour > 23)
 	return -1;
 
+    if (year < 50)
+      year += 100 ;
+    
     for (seconds = 0, year += 1900, i = 1970; i < year; i++)
 	seconds += DaysInYear(i);
     if (DaysInYear(year) == 366 && month > 2)
