@@ -1,4 +1,4 @@
-/*  $Revision: 1.30 $
+/*  $Revision: 1.32 $
 **
 **  Check article, send it to the local server.
 */
@@ -17,6 +17,7 @@
 #define FLUSH_ERROR(F)		(fflush((F)) == EOF || ferror((F)))
 #define HEADER_DELTA		20
 
+STATIC char     *tmpPtr ;
 STATIC char	Error[SMBUF];
 STATIC char	NGSEPS[] = NG_SEPARATOR;
 STATIC char	**OtherHeaders;
@@ -85,6 +86,60 @@ HEADER *EndOfTable = ENDOF(Table);
 
 
 
+/* Join() and MaxLength() are taken from innd.c */
+/*
+**  Turn any \r or \n in text into spaces.  Used to splice back multi-line
+**  headers into a single line.
+*/
+STATIC char *
+Join(text)
+    register char	*text;
+{
+    register char	*p;
+
+    for (p = text; *p; p++)
+	if (*p == '\n' || *p == '\r')
+	    *p = ' ';
+    return text;
+}
+
+/*
+**  Return a short name that won't overrun our bufer or syslog's buffer.
+**  q should either be p, or point into p where the "interesting" part is.
+*/
+char *
+MaxLength(p, q)
+    char		*p;
+    char		*q;
+{
+    static char		buff[80];
+    register int	i;
+
+    /* Already short enough? */
+    i = strlen(p);
+    if (i < sizeof buff - 1)
+	return Join(p);
+
+    /* Simple case of just want the begining? */
+    if (q - p < sizeof buff - 4) {
+	(void)strncpy(buff, p, sizeof buff - 4);
+	(void)strcpy(&buff[sizeof buff - 4], "...");
+    }
+    /* Is getting last 10 characters good enough? */
+    else if ((p + i) - q < 10) {
+	(void)strncpy(buff, p, sizeof buff - 14);
+	(void)strcpy(&buff[sizeof buff - 14], "...");
+	(void)strcpy(&buff[sizeof buff - 11], &p[i - 10]);
+    }
+    else {
+	/* Not in last 10 bytes, so use double elipses. */
+	(void)strncpy(buff, p, sizeof buff - 17);
+	(void)strcpy(&buff[sizeof buff - 17], "...");
+	(void)strncpy(&buff[sizeof buff - 14], &q[-5], 10);
+	(void)strcpy(&buff[sizeof buff - 4], "...");
+    }
+    return Join(buff);
+}
 /*
 **  Trim trailing spaces, return pointer to first non-space char.
 */
@@ -215,7 +270,7 @@ CheckControl(ctrl)
 	;
     else {
 	(void)sprintf(Error, "\"%s\" is not a valid control message",
-		ctrl);
+		MaxLength(ctrl,ctrl));
 	return Error;
     }
     *p = save;
@@ -238,7 +293,7 @@ CheckDistribution(p)
     do {
 	for (dp = BadDistribs; *dp; dp++)
 	    if (wildmat(p, *dp)) {
-		(void)sprintf(Error, "Illegal distribution \"%s\"", p);
+		(void)sprintf(Error, "Illegal distribution \"%s\"", MaxLength(p,p));
 		return Error;
 	    }
     } while ((p = strtok((char *)NULL, SEPS)) != NULL);
@@ -589,13 +644,25 @@ ValidNewsgroups(hdr, article)
 	    continue;
 	FoundOne = TRUE;
 	DDcheck(h, p);
+        if (PERMspecified) {
+            grplist[0] = p;
+            grplist[1] = NULL;
+            if (!PERMmatch(PERMdefault, PERMlist, grplist)) {
+                sprintf(Error,"%s: %s\r\n", NNTP_NOSUCHGROUP,p);
+                DISPOSE(groups);
+                tmpPtr = DDend(h);
+                DISPOSE(tmpPtr);
+                return Error;
+            }
+        }
 	switch (gp->Flag) {
 	case NF_FLAG_OK:
 	    break;
 	case NF_FLAG_MODERATED:
 	    if (!approved) {
 		DISPOSE(groups);
-		DISPOSE(DDend(h));
+                tmpPtr = DDend(h);
+		DISPOSE(tmpPtr);
 		return MailArticle(gp->Name, article);
 	    }
 	    break;
@@ -613,20 +680,14 @@ ValidNewsgroups(hdr, article)
 		    p, gp->Alias);
 	    break;
 	}
-        if (PERMspecified) {
-	    grplist[0] = p;
-	    grplist[1] = NULL;
-	    if (!PERMmatch(PERMdefault, PERMlist, grplist)) {
-		sprintf(Error,"%s: %s\r\n", NNTP_NOSUCHGROUP,p);
-	    }
-        }
     } while ((p = strtok((char *)NULL, NGSEPS)) != NULL);
     DISPOSE(groups);
 
     if (!FoundOne && !IsNewgroup)
-	(void)sprintf(Error, "No valid newsgroups in \"%s\"", hdr);
+	(void)sprintf(Error, "No valid newsgroups in \"%s\"", MaxLength(hdr,hdr));
     if (Error[0]) {
-	DISPOSE(DDend(h));
+        tmpPtr = DDend(h);
+	DISPOSE(tmpPtr) ;
 	return Error;
     }
 
@@ -689,44 +750,39 @@ Spoolit(article, Error)
     char		*Error;
 {
     static char		CANTSPOOL[NNTP_STRLEN+2];
-    static char		SPOOLNEWS[] = _PATH_SPOOLNEWS;
     register HEADER	*hp;
     register FILE	*F = NULL;
     register int	i;
     char		temp[BUFSIZ];
+    char		path[BUFSIZ];
     struct stat		Sb;
 
     /* Initialize the returned error message */
     sprintf(CANTSPOOL, "%s and can't write text to local spool file", Error);
 
     /* Try to write it to the spool dir. */
-    if (stat(SPOOLNEWS, &Sb) >= 0 && S_ISDIR(Sb.st_mode)) {
-	TempName(SPOOLNEWS, temp);
-	(void)umask(0);
-	if ((i = open(temp, O_WRONLY | O_CREAT, BATCHFILE_MODE)) >0 ) {
-            if ((F = fdopen(i, "w")) == NULL) {
-                close(i);
-            }
-        }
-        if (F==NULL) {
-            sprintf(CANTSPOOL, "%s and can't create local spool file", Error);
-            return CANTSPOOL;
-        }
+    TempName(_PATH_SPOOLNEWS, temp);
+    /* rnews -U ignores files starting with . */
+    strrchr(temp, '/')[1] = '.';
+    if ((F = fopen(temp, "w")) == NULL) {
+        syslog(L_FATAL, "cant open %s %m", temp);
+        return CANTSPOOL;
     }
+    fchmod(fileno(F), BATCHFILE_MODE);
 
     /* Write the headers and a blank line. */
     for (hp = Table; hp < ENDOF(Table); hp++)
 	if (hp->Value) {
 	    (void)fprintf(F, "%s: %s\n", hp->Name, hp->Value);
 	    if (FLUSH_ERROR(F)) {
-		(void)pclose(F);
+		(void)fclose(F);
 		return CANTSPOOL;
 	    }
 	}
     for (i = 0; i < OtherCount; i++) {
 	(void)fprintf(F, "%s\n", OtherHeaders[i]);
 	if (FLUSH_ERROR(F)) {
-	    (void)pclose(F);
+	    (void)fclose(F);
 	    return CANTSPOOL;
 	}
     }
@@ -740,13 +796,16 @@ Spoolit(article, Error)
     }
 
     /* Flush and catch any errors */
-    if (FLUSH_ERROR(F)) {
-	(void)fclose(F);
+    if (fclose(F))
+	return CANTSPOOL;
+
+    TempName(_PATH_SPOOLNEWS, path);
+    if (rename(temp, path) < 0) {
+        syslog(L_FATAL, "cant rename %s %s %m", temp, path);
 	return CANTSPOOL;
     }
 
     /* Article has been spooled */
-    fclose(F);
     return NULL;
 }
 
@@ -791,10 +850,17 @@ ARTpost(article, idbuff)
 	return error;
     if (i == 0 && HDR(_control) == NULL)
 	return "Article is empty";
+
+    if (idbuff != NULL)
+      strcpy (idbuff,"(mailed to moderator)") ;
     if ((error = ValidNewsgroups(HDR(_newsgroups), article)) != NULL
      || WasMailed)
 	return error;
-    strcpy(frombuf, HDR(_from));
+    if (idbuff != NULL)
+      idbuff [0] = '\0' ;
+    
+    strncpy(frombuf, HDR(_from), sizeof(frombuf) - 1);
+    frombuf[sizeof(frombuf) - 1] = '\0';
     HeaderCleanFrom(frombuf);
     p = strchr(frombuf, '@');
     if (p) {
@@ -903,7 +969,9 @@ ARTpost(article, idbuff)
 
     /* Send a quit and close down */
     SendQuit(FromServer, ToServer);
-    if (idbuff)
-	(void)strcpy(idbuff, HDR(_messageid));
+    if (idbuff) {
+	(void)strncpy(idbuff, HDR(_messageid), SMBUF - 1);
+	idbuff[SMBUF - 1] = '\0';
+    }
     return NULL;
 }
