@@ -132,6 +132,9 @@ set inn_sed "/usr/bin/sed"
 ##  =()<set inn_sort "@<_PATH_SORT>@">()=
 set inn_sort "/usr/bin/sort"
 
+##  =()<set inn_sendmail "@<_PATH_SENDMAIL>@">()=
+set inn_sendmail "/usr/sbin/sendmail -oi %s"
+
 ##  =()<set inn_serverpid "@<_PATH_SERVERPID>@">()=
 set inn_serverpid "/var/news/run/innd.pid"
 
