@@ -1,5 +1,5 @@
 #! /bin/sh
-##  $Revision: 1.1 $
+##  $Revision: 1.5 $
 ##  Run subst over a list of files.  Get files in and out of source control
 ##  if present.
 
@@ -7,8 +7,7 @@
 MODE=none
 PLAIN="$*"
 CONTROLLED=
-BASEPWD=`pwd`
-BASEDIRNAME=`basename $BASEPWD`
+BASEDIRNAME=`basename \`pwd\``
 
 if [ -d RCS -a ! -d SCCS ] ; then
     # Find out which files are under RCS control.
@@ -16,7 +15,7 @@ if [ -d RCS -a ! -d SCCS ] ; then
     PLAIN=
     for I
     do
-	if [ -f RCS/${I},v ] ; then
+	if [ -f RCS/${I},v -o -f RCS/${I} ] ; then
 	    CONTROLLED="${CONTROLLED} ${I}"
 	else
 	    PLAIN="${PLAIN} ${I}"
@@ -65,7 +64,7 @@ for I in ${CONTROLLED} ${PLAIN} ; do
 	    test -f SCCS/s.${I} && sccs unedit ${I}
 	    ;;
 	rcs)
-	    test -f $RCS/${I},v && rcs -q -u ${I} && ci -u -q -f ${I}
+	    test -f RCS/${I},v -o -f RCS/${I} && ci -u -q ${I}
 	    ;;
 	esac
     else
